function A = read_egi_simple_bin(filename, bs, ChannelNames, time0code)

%A = read_egi_simple_bin(filename, bs)
%
%A is a structure containing the data from the file filename
%set bs to 1 to enable byte-swapping, -1 to guess
%ChannelNames is a cell array of the strings to assign to the different
%channels
% time0code is the code which specifies where time 0 is (for display
% purposes)
% if time0code is a number, then it specifies the time offset of the first
% sample

% written by Drew Morris (drew.j.morris@gmail.com)
% September 2010

A.type=0;

%check number of arguments given
if (nargin < 2)
    bs=0;
end


%open the file
f = fopen(filename);

if (f == -1)
    error(['The file "'   filename  '" was not found.'])
end

%byte swapping

if (bs==-1)
    %guess byte swapping
    %data probably came from Big-Endian system
    [s m endian]=computer;
    if endian=='B'
        bs=0;
    else
        bs=1;
    end
end

if (bs==1)
    xx = fread(f,1,'int32');    % xx = 83886080
    yy = int32(xx);             % converts the elements of an array into 
                                % signed 32-bit (4-byte) integers of class int32. 
    A.type=swapbytes(yy);       % converting little-endian values to big-endian.
                                % A.type = 5
else
    A.type=fread(f,1,'int32');
end

switch A.type
    case 2
        A.typeString='Continuous signed integer';
        precision='int32';
        A.segmented=0;
    case 3
        A.typeString='Segmented signed integer';
        precision='int32';
        A.segmented=1;
    case 4
        A.typeString='Continuous single';
        precision='single';
        A.segmented=0;
    case 5
        %segmented
        A.typeString='Segmented single';
        precision='single';
        A.segmented=1;
    case 6
        A.typeString='Continuous double';
        precision='double';
        A.segmented=0;
    case 7
        A.typeString='Segmented double';
        precision='double';
        A.segmented=1;
    otherwise
        fclose(f);
        error(['File type is ' int2str(A.type) '. Check the file or try byteswapping']);
end

%define the functions to read from the file

if (bs==1)
    %byte swap the data as they come out
    readshort=@(F,N) swapbytes(int16(fread(F,N,'int16')));
    readlong=@(F,N) swapbytes(int32(fread(F,N,'int32')));
    readfloat=@(F,N) swapbytes(cast(fread(F,N,precision),precision));
else
    %read as normal
    readshort=@(F,N) fread(F,N,'int16');
    readlong=@(F,N) fread(F,N,'int32');
    readfloat=@(F,N) fread(F,N,precision);
   
end


%taken from the EGI "NetStation File Formats"

A.timestamp.year=readshort(f,1);
A.timestamp.month=readshort(f,1);
A.timestamp.day=readshort(f,1);
A.timestamp.hh=readshort(f,1);
A.timestamp.mm=readshort(f,1);
A.timestamp.ss=readshort(f,1);
A.timestamp.ms=readlong(f,1);
A.timestamp.dateString=sprintf('%d/%02d/%02d %02d:%02d:%02d.%03d', A.timestamp.year, A.timestamp.month, A.timestamp.day, A.timestamp.hh, A.timestamp.mm, A.timestamp.ss, A.timestamp.ms);
A.sampleRate=readshort(f,1);
A.channels=readshort(f,1);
A.boardGain=readshort(f,1);
A.bits=readshort(f,1);
A.fsV=readshort(f,1);
if (A.type==3 || A.type==5 || A.type==7)
    %segmented file. Read in segment info
    catNames=readshort(f,1);
    for x =1:catNames
        strLen=fread(f,1,'uchar');
        A.categoryName(x)={char(fread(f,strLen,'char')')};
    end
    A.segments=readshort(f,1);
    A.segmentLength=readlong(f,1);
    A.samples=A.segmentLength;
else
%continuous
    A.segments=1;
    A.samples=readlong(f,1);
    A.segmentLength=A.samples;
end


A.eventCodes=readshort(f,1);
for x = 1:A.eventCodes
    A.eventCode(x)={char(fread(f,4,'char')')};
end


if (A.segmented==1)
    for x = 1:A.segments
        A.segment(x).catIndex=readshort(f,1);
        if (A.segment(x).catIndex > 0 && A.segment(x).catIndex <= length(A.categoryName))
            A.segment(x).category=A.categoryName{A.segment(x).catIndex};
        end
        A.segment(x).timestamp=readlong(f,1);
        datawidth=double(A.channels+A.eventCodes); % 131 = 129 + 2
        A.segment(x).channelData=readfloat(f,datawidth*double(A.segmentLength));
        A.segment(x).channelData=reshape(A.segment(x).channelData, [datawidth, A.segmentLength]);
        A.segment(x).eventData=A.segment(x).channelData(A.channels+1:A.channels+A.eventCodes,:);
        A.segment(x).channelData=A.segment(x).channelData(1:A.channels,:);

    end
else
    datawidth=double(A.channels+A.eventCodes);
    A.channelData=readfloat(f,datawidth*double(A.samples));
    A.channelData=reshape(A.channelData, [datawidth, A.samples]);
    A.eventData=A.channelData(A.channels+1:A.channels+A.eventCodes,:);
    A.channelData=A.channelData(1:A.channels,:);
end

fclose(f);


if (nargin > 2)
    %add in channel names
    if length(ChannelNames) ~= A.channels
        disp('Warning: ChannelNames is not the same size as number of channels in data file. Populating with channel numbers');
        for x=1:A.channels
            A.channelName{x}= sprintf('%d', x);
        end
    else
       
        sizechan=size(ChannelNames);
        if sizechan(1)==1
            A.channelName = ChannelNames;
        else
            A.channelName = ChannelNames';
        end
        
    end
else
    disp('Warning: ChannelNames were not provided. Populating with channel numbers');
    for x=1:A.channels
        A.channelName(x)=sprritnf('%d', x);
    end
end

if (nargin < 4)
    time0code=0;
end


if (ischar(time0code))
    %find the onset of this code in the data
    eventIndex=find(strcmp(A.eventCode,time0code), 1);
    if (isempty(eventIndex))
        warning(['The specified time0code, "' time0code '", was not found. Using time offset of 0']);
        if (A.segmented==0) 
            A.timeData=0:(1/double(A.sampleRate)):(double(A.samples-1)/double(A.sampleRate));
        else
            for x=1:A.segments
                A.segment(x).timeData=0:(1/double(A.sampleRate)):(double(A.samples-1)/double(A.sampleRate));
            end
        end
    else
        if (A.segmented==0)
            time0index=find(A.eventData(eventIndex,:)==1,1);
            if (isempty(time0index))
                A.timeData=0:(1/double(A.sampleRate)):((double(A.samples)-1)/double(A.sampleRate));
            else
                t0 = -(time0index-1)/double(A.sampleRate);
                A.timeData=t0:(1/double(A.sampleRate)):(t0+(double(A.samples)-1)/double(A.sampleRate));
            end
        else
            for x=1:A.segments
                time0index=find(A.segment(x).eventData(eventIndex,:)==1,1);
                if (isempty(time0index))
                    A.segment(x).timeData=0:(1/double(A.sampleRate)):((double(A.samples)-1)/double(A.sampleRate));
                else
                    t0 = -(time0index-1)/double(A.sampleRate);
                    A.segment(x).timeData=t0:(1/double(A.sampleRate)):(t0+(double(A.samples)-1)/double(A.sampleRate));
                end
            end
        end
    end
else
    %its a number, set time to start at this number
    if (A.segmented==0)
        A.timeData=time0code:(1/double(A.sampleRate)):(time0code+(double(A.samples)-1)/double(A.sampleRate));
    else
        for x=1:A.segments
             A.segment(x).timeData=time0code:(1/double(A.sampleRate)):(time0code+(double(A.samples)-1)/double(A.sampleRate));
        end
    end
    
end

% disp(['Data read from file: ' filename ]);
% disp(['File type: ' A.typeString]);
% disp(['Channels: ' num2str(A.channels)]);
% if (A.segmented==1)
%     disp(['Segments: ' num2str(A.segments)]);
%     disp(['Samples/Segment: ' num2str(A.segmentLength)]);
% else
%     disp(['Samples: ' num2str(A.samples)]);
% end

A.filename=filename;